#import "MarkdownParserTestCases.h"
#import "MDMarkdownParser.h"

@implementation MarkdownParserTestCases

-(void)setUp{
	parser = [[MDMarkdownParser alloc]init];
}


-(void)tearDown{
	[parser release];
}


#pragma mark DETABIFY
-(void)testDetabifyWorks{
	test = @"hello world";
	test = [parser detabString:test];
	STAssertNotNil(test, @"Parser: %@", parser);
}


-(void)testDetabifyBeginingOfLine{
	test = @"\thello world\n";
	test = [parser detabString:test];
	STAssertEqualObjects(test, @"    hello world\n", @"one tab");
	
	test = @"\t\t\thello world\n";
	test = [parser detabString:test];
	STAssertEqualObjects(test, @"            hello world\n", @"three tabs");
	
	test = @"hello world";
	test = [parser detabString:test];
	STAssertEqualObjects(test, @"hello world\n", @"adds newlines.");
}


-(void)testDetabifyMiddleOfLine{
	test = @"h\tello world.\n";
	test = [parser detabString:test];
	STAssertEqualObjects(test, @"h   ello world.\n", @"three break");
	
	test = @"he\tllo world.\n";
	test = [parser detabString:test];
	STAssertEqualObjects(test, @"he  llo world.\n", @"two break");

	test = @"hel\tlo world.\n";
	test = [parser detabString:test];
	STAssertEqualObjects(test, @"hel lo world.\n", @"one break");
	
	test = @"hell\to world.\n";
	test = [parser detabString:test];
	STAssertEqualObjects(test, @"hell    o world.\n", @"four break");
	
}


-(void)testDetabifyCombination{
	test = @"\th\tel\tlo world.\n";
	test = [parser detabString:test];
	STAssertEqualObjects(test, @"    h   el  lo world.\n", @"combo");
}



#pragma WHITESPACE ONLY LINES
-(void)testCleanWhitespaceOnlyLinesWorks{
	base = @"Some thing who cares.\nWoooo!\n";
	test = [parser cleanWhitespaceOnlyLinesInString:base];
	STAssertNotNil(test, @"parser: %@", parser);
}


-(void)testCleanWhitespaceOnlyLines{
	test = @"   \n";
	test = [parser cleanWhitespaceOnlyLinesInString:test];
	STAssertEqualObjects(test, @"\n", @"spaces");

	test = @"\n";
	test = [parser cleanWhitespaceOnlyLinesInString:test];
	STAssertEqualObjects(test, @"\n", @"newline");
	
	test = @"hello\n   \nworld\n";
	test = [parser cleanWhitespaceOnlyLinesInString:test];
	STAssertEqualObjects(test, @"hello\n\nworld\n", @"newline");
	
	base=@" a \n";
	test=[parser cleanWhitespaceOnlyLinesInString:base];
	STAssertEqualObjects(test, base, @"shouldn't be touched");
}

@end
